@@ -55,7 +55,7 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)res_comp.c	8.1 (Berkeley) 6/4/93";
-static char rcsid[] = "res_comp.c,v 1.2 1995/05/06 14:12:40 hjl Exp";
+static char rcsid[] = "$Id: res_comp.c,v 1.5 1996/10/04 22:52:52 hjl Exp $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/param.h>
@@ -72,10 +72,32 @@ static char rcsid[] = "res_comp.c,v 1.2 1995/05/06 14:12:40 hjl Exp";
 #pragma weak dn_skipname = __dn_skipname
 #pragma weak putlong = __putlong
 #pragma weak putshort = __putshort
-#endif
+#endif	/* __ELF__ */
 
-static int	dn_find __P((u_char *exp_dn, u_char *msg,
-			     u_char **dnptrs, u_char **lastdnptr));
+#ifdef __STDC__
+int         dn_expand    ( const u_char *, const u_char *, const u_char *,
+                      char *, int );
+int         dn_comp      ( const char *, u_char *, int, u_char **, u_char ** );
+static int  mklower      ( register int );
+static int  dn_find      ( u_char *, u_char *, u_char **, u_char ** );
+u_int16_t   _getshort    ( register const u_char * );
+u_int32_t   _getlong     ( register const u_char * );
+#ifndef __GNUC__
+int         __dn_skipname( const u_char *, const u_char * );
+void        __putshort   ( register u_int16_t, register u_char * );
+void        __putlong    ( register u_int32_t, register u_char * );
+#endif   /* __GNUC__ */
+#else    /* __STDC__ */
+int         dn_expand     ();
+int         dn_comp       ();
+int         __dn_skipname ();
+static int  mklower       ();
+static int  dn_find       ();
+u_int16_t   _getshort     ();
+u_int32_t   _getlong      ();
+void        __putshort    ();
+void        __putlong     ();
+#endif	/* __STDC__ */
 
 /*
  * Expand compressed domain name 'comp_dn' to full domain name.
@@ -84,11 +106,18 @@ static int	dn_find __P((u_char *exp_dn, u_char *msg,
  * 'exp_dn' is a pointer to a buffer of size 'length' for the result.
  * Return size of compressed name or -1 if there was an error.
  */
+
+#ifdef __STDC__
+int
+dn_expand( const u_char *msg, const u_char *eomorig, const u_char *comp_dn,
+	char *exp_dn, int length )
+#else
 int
 dn_expand(msg, eomorig, comp_dn, exp_dn, length)
 	const u_char *msg, *eomorig, *comp_dn;
 	char *exp_dn;
 	int length;
+#endif
 {
 	register const u_char *cp;
 	register char *dn;
@@ -102,7 +131,7 @@ dn_expand(msg, eomorig, comp_dn, exp_dn, length)
 	/*
 	 * fetch next label in domain name
 	 */
-	while (n = *cp++) {
+	while ((n = *cp++)) {
 		/*
 		 * Check for indirection
 		 */
@@ -169,11 +198,18 @@ dn_expand(msg, eomorig, comp_dn, exp_dn, length)
  * If 'dnptr' is NULL, we don't try to compress names. If 'lastdnptr'
  * is NULL, we don't update the list.
  */
+
+#ifdef __STDC__
+int
+dn_comp( const char *exp_dn, u_char *comp_dn, int length, u_char **dnptrs,
+		  u_char **lastdnptr )
+#else
 int
 dn_comp(exp_dn, comp_dn, length, dnptrs, lastdnptr)
 	const char *exp_dn;
 	u_char *comp_dn, **dnptrs, **lastdnptr;
 	int length;
+#endif
 {
 	register u_char *cp, *dn;
 	register int c, l;
@@ -249,9 +285,15 @@ dn_comp(exp_dn, comp_dn, length, dnptrs, lastdnptr)
 /*
  * Skip over a compressed domain name. Return the size or -1.
  */
+
+#ifdef __STDC__
+int
+__dn_skipname( const u_char *comp_dn, const u_char *eom )
+#else
 int
 __dn_skipname(comp_dn, eom)
 	const u_char *comp_dn, *eom;
+#endif
 {
 	register const u_char *cp;
 	register int n;
@@ -278,9 +320,14 @@ __dn_skipname(comp_dn, eom)
 	return (cp - comp_dn);
 }
 
+#ifdef __STDC__
+static int
+mklower( register int ch )
+#else
 static int
 mklower(ch)
 	register int ch;
+#endif
 {
 	if (isascii(ch) && isupper(ch))
 		return (tolower(ch));
@@ -293,10 +340,16 @@ mklower(ch)
  * dnptrs is the pointer to the first name on the list,
  * not the pointer to the start of the message.
  */
+
+#ifdef __STDC__
+static int
+dn_find( u_char *exp_dn, u_char *msg, u_char **dnptrs, u_char **lastdnptr )
+#else
 static int
 dn_find(exp_dn, msg, dnptrs, lastdnptr)
 	u_char *exp_dn, *msg;
 	u_char **dnptrs, **lastdnptr;
+#endif
 {
 	register u_char *dn, *cp, **cpp;
 	register int n;
@@ -305,7 +358,7 @@ dn_find(exp_dn, msg, dnptrs, lastdnptr)
 	for (cpp = dnptrs; cpp < lastdnptr; cpp++) {
 		dn = exp_dn;
 		sp = cp = *cpp;
-		while (n = *cp++) {
+		while ((n = *cp++)) {
 			/*
 			 * check for indirection
 			 */
@@ -344,9 +397,14 @@ dn_find(exp_dn, msg, dnptrs, lastdnptr)
  * Routines to insert/extract short/long's.
  */
 
+#ifdef __STDC__
+u_int16_t
+_getshort( register const u_char *msgp )
+#else
 u_int16_t
 _getshort(msgp)
 	register const u_char *msgp;
+#endif
 {
 	register u_int16_t u;
 
@@ -354,10 +412,14 @@ _getshort(msgp)
 	return (u);
 }
 
-
+#ifdef __STDC__
+u_int32_t
+_getlong( register const u_char *msgp )
+#else
 u_int32_t
 _getlong(msgp)
 	register const u_char *msgp;
+#endif
 {
 	register u_int32_t u;
 
@@ -365,10 +427,11 @@ _getlong(msgp)
 	return (u);
 }
 
-void
 #if defined(__STDC__) || defined(__cplusplus)
+void
 __putshort(register u_int16_t s, register u_char *msgp)	/* must match proto */
 #else
+void
 __putshort(s, msgp)
 	register u_int16_t s;
 	register u_char *msgp;
@@ -377,10 +440,15 @@ __putshort(s, msgp)
 	PUTSHORT(s, msgp);
 }
 
+#ifdef __STDC__
+void
+__putlong( register u_int32_t l, register u_char *msgp )
+#else
 void
 __putlong(l, msgp)
 	register u_int32_t l;
 	register u_char *msgp;
+#endif
 {
 	PUTLONG(l, msgp);
 }
