@@ -40,23 +40,35 @@ static char sccsid[] = 	"@(#)openchild.c	2.3 88/08/15 4.0 RPCSRC; from 1.7 88/02
  * interface, but for internal use only!
  */
 #include <stdio.h>
+#include <unistd.h>
+#include <rpc/clnt.h>
+#include <malloc.h>
+#include <string.h>
 
-extern char *malloc();
-extern char *rindex();
-extern int sprintf();
-
-extern char *__basename();
 #define basename __basename
 static char SHELL[] = "/bin/sh";
 
+#ifdef __STDC__
+int          _openchild   ( char *, FILE **, FILE ** );
+static char *basename     ( char * );
+#else	/* ! __STDC__ */
+int _openchild();
+static char *basename();
+#endif	/* __STDC__ */
 
 /*
  * returns pid, or -1 for failure
  */
+#ifdef __STDC__
+int
+_openchild( char *command, FILE **fto, FILE **ffrom )
+#else
+int
 _openchild(command, fto, ffrom)
 	char *command;
 	FILE **fto;
 	FILE **ffrom;
+#endif
 {
 	int i;
 	int pid;
@@ -119,10 +131,14 @@ error1:
 	return (-1);
 }
 
-#if 0
+#ifdef __STDC__
+static char *
+basename( char *path )
+#else
 static char *
 basename(path)
 	char *path;
+#endif
 {
 	char *p;
 
@@ -133,4 +149,3 @@ basename(path)
 		return (p + 1);
 	}
 }
-#endif
