@@ -31,9 +31,11 @@
  *  Modifications for linux by Swen Thuemmler <swen@uni-paderborn.de>
  */
 
+#if 0
 #ifndef LINT
 static char *rcsid = "yplib.c,v 2.6 1994/05/27 14:34:43 swen Exp";
 #endif
+#endif
 
 #include <sys/param.h>
 #include <sys/types.h>
@@ -43,30 +45,72 @@ static char *rcsid = "yplib.c,v 2.6 1994/05/27 14:34:43 swen Exp";
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
-#include "yp.h"
+#include <rpcsvc/yp_prot.h>
 #include <rpcsvc/ypclnt.h>
 
 #ifndef BINDINGDIR
 #define BINDINGDIR "/var/yp/binding"
 #endif
 #define YPMATCHCACHE
-#define USE_BINDINGDIR 0 /* We don't want it */
+#define USE_BINDINGDIR 1
 
-static void ypmatch_add (char *, char *, int, char *, int);
-static bool_t ypmatch_find (char *, char *, int, char **, int *);
-int __yp_dobind (char *, struct dom_binding **);
-static void __yp_unbind (struct dom_binding *);
-int __yp_check (char **);
-int yp_maplist (char *, struct ypmaplist **);
-int (*ypresp_allfn)(int, char *, int, char *, int, char *);
-void *ypresp_data;
+/*
+ * End of Prototypes
+ */
+
+struct dom_binding {       
+	struct dom_binding *dom_pnext;
+	char dom_domain[YPMAXDOMAIN + 1];
+	struct sockaddr_in dom_server_addr;
+	unsigned short int dom_server_port;
+	int dom_socket;      
+	CLIENT *dom_client;  
+	unsigned short int dom_local_port;
+	long int dom_vers;   
+};
+#ifdef YPMATCHCACHE
+static void     ypmatch_add             (char *, char *, int, char *, int);
+static bool_t   ypmatch_find            (char *, char *, int, char **, int *);
+#endif
+int             __yp_dobind             (char *, struct dom_binding **);
+static void     __yp_unbind             (struct dom_binding *);
+int             yp_bind                 (char *);
+void            yp_unbind               (char *);
+int             yp_match                (char *, char *, char *, int,
+                                         char **, int *);
+int             yp_get_default_domain   (char **);
+int             yp_first                (char *, char *, char **, int *,
+                                         char **, int *);
+int             yp_next                 (char *, char *, char *, int, char **,
+                                         int *, char **, int *);
+int             yp_all                  (char *, char *,
+                                         struct ypall_callback *);
+int             yp_order                (char *, char *, int *);
+int             yp_master               (char *, char *, char **);
+int             yp_maplist              (char *, struct ypmaplist **);
+char           *yperr_string            (int);
+int             ypprot_err              (long int);
+int             __yp_check              (char **);
+
+/*
+ * End of Prototypes
+ */
 
 struct dom_binding *__ypbindlist;
 static char __yp_domain[MAXHOSTNAMELEN];
-int __yplib_timeout = 10;
+int const __yplib_timeout = 10;
+int const __yplib_maxtries = 5;
+
+/* Added by vch. These two variables are not defined elsewhere. */
+/* This is rather dangerous coding. The address of 'ypresp' is assigned
+ * in 'yp_all()' from 'struct ypall_callback.foreach'.  There are no checks
+ * here or in 'getpwent()' that 'ypresp_allfn()' is inialized. vch */
+
+int (*ypresp_allfn) (int, char *, int, char *, int, char *);
+char *ypresp_data;
 
 #ifdef YPMATCHCACHE
-int __yplib_cache = 5;
+int const __yplib_cache = 5;
 
 static struct ypmatch_ent
 {
@@ -185,7 +229,7 @@ __yp_dobind (char *dom, struct dom_binding **ypdb)
   int new = 0, r;
   
   gpid = getpid ();
-  if (!(-1 == pid || gpid == pid))
+  if (-1 != pid && gpid != pid)
     {
       ysd = __ypbindlist;
       while (ysd)
@@ -221,67 +265,52 @@ __yp_dobind (char *dom, struct dom_binding **ypdb)
 #if USE_BINDINGDIR
   if (0 == ysd->dom_vers)
     {
+      struct iovec iov[2];
+      struct ypbind_resp ybr;
+      u_short ypb_port;
+
       sprintf (path, "%s/%s.%d", BINDINGDIR, dom, 2);
-      if ((fd = open (path, O_RDONLY)) == -1)
+      fd = open (path, O_RDONLY);
+      if (fd < 0)
+              /* no binding file, try RPC */
+        goto try_ypbind;
+        
+      iov[0].iov_base = (caddr_t) & ypb_port;
+      iov[0].iov_len = sizeof ypb_port;
+      iov[1].iov_base = (caddr_t) & ybr;
+      iov[1].iov_len = sizeof ybr;
+      
+      r = readv (fd, iov, 2);
+      if (r != iov[0].iov_len + iov[1].iov_len)
         {
-              /* no binding file, YP is dead. */
-          if (new)
-            free (ysd);
-          return YPERR_YPBIND;
-        }
-      if (flock (fd, LOCK_EX | LOCK_NB) == -1 && errno == EWOULDBLOCK)
-        {
-          struct iovec iov[2];
-          struct ypbind_resp ybr;
-          u_short ypb_port;
-          
-          iov[0].iov_base = (caddr_t) & ypb_port;
-          iov[0].iov_len = sizeof ypb_port;
-          iov[1].iov_base = (caddr_t) & ybr;
-          iov[1].iov_len = sizeof ybr;
-          
-          r = readv (fd, iov, 2);
-          if (r != iov[0].iov_len + iov[1].iov_len)
-            {
-              close (fd);
-              ysd->dom_vers = -1;
-              goto again;
-            }
-          
-          bzero (&ysd->dom_server_addr, sizeof ysd->dom_server_addr);
-          ysd->dom_server_addr.sin_family = AF_INET;
-          ysd->dom_server_addr.sin_addr.s_addr =
-            ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr.s_addr;
-          ysd->dom_server_addr.sin_port =
-            ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
-          ysd->dom_server_port = ysd->dom_server_addr.sin_port;
           close (fd);
-          goto gotit;
-        }
-      else
-        {
-              /* no lock on binding file, YP is dead. */
-          close (fd);
-          if (new)
-            free (ysd);
-          return YPERR_YPBIND;
+          ysd->dom_vers = -1;
+          goto again;
         }
+      
+      bzero (&ysd->dom_server_addr, sizeof ysd->dom_server_addr);
+      ysd->dom_server_addr.sin_family = AF_INET;
+      ysd->dom_server_addr.sin_addr.s_addr =
+        ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr.s_addr;
+      ysd->dom_server_addr.sin_port =
+        ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
+      ysd->dom_server_port = ysd->dom_server_addr.sin_port;
+      close (fd);
+      goto gotit;
     }
 #endif
+  try_ypbind:
   if (-1 == ysd->dom_vers || 0 == ysd->dom_vers)
     {
       bzero ((char *) &clnt_sin, sizeof clnt_sin);
       clnt_sin.sin_family = AF_INET;
-      clnt_sin.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
+      clnt_sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
       clnt_sock = RPC_ANYSOCK;
       client = clnttcp_create (&clnt_sin,
                                YPBINDPROG, YPBINDVERS,
                                &clnt_sock, 0, 0);
       if (NULL == client)
         {
-#if 0
-          clnt_pcreateerror ("clnttcp_create");
-#endif
           if (new)
             free (ysd);
           return YPERR_YPBIND;
@@ -290,22 +319,19 @@ __yp_dobind (char *dom, struct dom_binding **ypdb)
       tv.tv_usec = 0;
       r = clnt_call (client,
                      YPBINDPROC_DOMAIN,
-                     xdr_domainname, dom,
-                     xdr_ypbind_resp, &ypbr,
+                     (xdrproc_t)xdr_domainname, dom,
+                     (xdrproc_t)xdr_ypbind_resp, (char *)&ypbr,
                      tv);
       if (r != RPC_SUCCESS)
         {
-          fprintf (stderr,
-                   "YP: server for domain %s not responding, still trying\n",
-                   dom);
           clnt_destroy (client);
-          ysd->dom_vers = -1;
-          goto again;
+          if (new)
+            free (ysd);
+          return YPERR_YPBIND;
         }
       clnt_destroy (client);
-      switch (ypbr.ypbind_status)
+      if (YPBIND_SUCC_VAL != ypbr.ypbind_status)
         {
-        case YPBIND_FAIL_VAL:
           switch(ypbr.ypbind_respbody.ypbind_error)
             {
             case YPBIND_ERR_ERR:
@@ -314,7 +340,6 @@ __yp_dobind (char *dom, struct dom_binding **ypdb)
             case YPBIND_ERR_NOSERV:
               fprintf(stderr,
                       "YPBINDPROC_DOMAIN: No bound server for domain %s\n",
-
                       dom);
               break;
             case YPBIND_ERR_RESC:
@@ -324,18 +349,20 @@ __yp_dobind (char *dom, struct dom_binding **ypdb)
               fprintf(stderr, "YPBINDPROC_DOMAIN: Unknown error\n");
               break;
             }
+          if (new)
+            free (ysd);
           return YPERR_DOMAIN;
-        case YPBIND_SUCC_VAL:
-          {
-            bzero ((char *) &ysd->dom_server_addr, sizeof ysd->dom_server_addr);
-            ysd->dom_server_addr.sin_family = AF_INET;
-            ysd->dom_server_addr.sin_port = 
-              ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
-            ysd->dom_server_addr.sin_addr.s_addr =
-              ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr.s_addr;
-            ysd->dom_server_port = 
-              ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
-          }
+        }
+      else
+        {
+          bzero ((char *) &ysd->dom_server_addr, sizeof ysd->dom_server_addr);
+          ysd->dom_server_addr.sin_family = AF_INET;
+          ysd->dom_server_addr.sin_port = 
+            ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
+          ysd->dom_server_addr.sin_addr.s_addr =
+            ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr.s_addr;
+          ysd->dom_server_port = 
+            ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
         }
       gotit:
       ysd->dom_vers = YPVERS;
@@ -351,9 +378,6 @@ __yp_dobind (char *dom, struct dom_binding **ypdb)
                                     tv, &ysd->dom_socket);
   if (NULL == ysd->dom_client)
     {
-#if 0
-      clnt_pcreateerror ("clntudp_create");
-#endif
       ysd->dom_vers = -1;
       goto again;
     }
@@ -408,30 +432,47 @@ yp_unbind (char *dom)
   return;
 }
 
+#ifdef __STDC__
 int
-yp_match (char *indomain, char *inmap, char *inkey, int inkeylen, char **outval, int *outvallen)
+yp_match ( char *indomain, char *inmap, char *inkey, int inkeylen,
+	  char **outval, int *outvallen)
+#else
+int
+yp_match (char *indomain,
+	  char *inmap,
+	  char *inkey,
+	  int inkeylen,
+	  char **outval,
+	  int *outvallen)
+#endif
 {
   struct dom_binding *ysd;
   struct ypresp_val yprv;
   struct timeval tv;
   struct ypreq_key yprk;
-  int r;
-  
+  int result, tries = 0;
+
+  if (NULL == indomain || '\0' == *indomain ||
+      NULL == inmap || '\0' == *inmap ||
+      NULL == inkey || 0 == inkeylen || '\0' == *inkey)
+    return YPERR_BADARGS;
+
   *outval = NULL;
   *outvallen = 0;
   
   again:
-  if (__yp_dobind (indomain, &ysd) != 0)
-    return YPERR_DOMAIN;
+  result = __yp_dobind(indomain, &ysd);
+  if (0 != result)
+    return result;
   
 #ifdef YPMATCHCACHE
-  if (!strcmp (__yp_domain, indomain)
-      && ypmatch_find (inmap, inkey,
-                       inkeylen, &yprv.valdat.dptr, &yprv.valdat.dsize))
+  if (!strcmp(__yp_domain, indomain)
+      && ypmatch_find (inmap, inkey, inkeylen,
+                       &yprv.valdat.valdat_val, &yprv.valdat.valdat_len))
     {
-      *outvallen = yprv.valdat.dsize;
+      *outvallen = yprv.valdat.valdat_len;
       *outval = (char *) malloc (*outvallen + 2);
-      bcopy (yprv.valdat.dptr, *outval, *outvallen);
+      bcopy (yprv.valdat.valdat_val, *outval, *outvallen);
       (*outval)[*outvallen] = '\n';
       (*outval)[*outvallen + 1] = '\0';
       return 0;
@@ -443,26 +484,30 @@ yp_match (char *indomain, char *inmap, char *inkey, int inkeylen, char **outval,
   
   yprk.domain = indomain;
   yprk.map = inmap;
-  yprk.keydat.dptr = inkey;
-  yprk.keydat.dsize = inkeylen;
+  yprk.keydat.keydat_val = inkey;
+  yprk.keydat.keydat_len = inkeylen;
   
   bzero ((char *) &yprv, sizeof yprv);
   
-  r = clnt_call (ysd->dom_client, YPPROC_MATCH,
-                 xdr_ypreq_key, &yprk,
-                 xdr_ypresp_val, &yprv,
-                 tv);
-  if (r != RPC_SUCCESS)
+  result = clnt_call (ysd->dom_client, YPPROC_MATCH,
+                      (xdrproc_t)xdr_ypreq_key, (char *)&yprk,
+                      (xdrproc_t)xdr_ypresp_val, (char *)&yprv,
+                      tv);
+  if (RPC_SUCCESS != result)
     {
       clnt_perror (ysd->dom_client, "yp_match: clnt_call");
       ysd->dom_vers = -1;
-      goto again;
+      __yp_unbind (ysd);
+      if (tries++ < __yplib_maxtries)
+        goto again;
+      return YPERR_RPC;
     }
-  if (!(r = ypprot_err (yprv.status)))
+  result = ypprot_err (yprv.status);
+  if (0 == result)
     {
-      *outvallen = yprv.valdat.dsize;
+      *outvallen = yprv.valdat.valdat_len;
       *outval = (char *) malloc (*outvallen + 2);
-      bcopy (yprv.valdat.dptr, *outval, *outvallen);
+      bcopy (yprv.valdat.valdat_val, *outval, *outvallen);
       (*outval)[*outvallen] = '\n';
       (*outval)[*outvallen + 1] = '\0';
 #ifdef YPMATCHCACHE
@@ -470,9 +515,9 @@ yp_match (char *indomain, char *inmap, char *inkey, int inkeylen, char **outval,
         ypmatch_add (inmap, inkey, inkeylen, *outval, *outvallen);
 #endif
     }
-  xdr_free (xdr_ypresp_val, (char *) &yprv);
+  xdr_free ((xdrproc_t)xdr_ypresp_val, (char *) &yprv);
   __yp_unbind (ysd);
-  return r;
+  return result;
 }
 
 int
@@ -487,17 +532,23 @@ yp_get_default_domain (char **domp)
 }
 
 int
-yp_first (char *indomain, char *inmap, char **outkey, int *outkeylen, char **outval, int *outvallen)
+yp_first (char *indomain, char *inmap, char **outkey, int *outkeylen,
+          char **outval, int *outvallen)
 {
   struct ypresp_key_val yprkv;
   struct ypreq_nokey yprnk;
   struct dom_binding *ysd;
   struct timeval tv;
-  int r;
+  int result = 0;
+  int tries = 0;
 
   *outkey = *outval = NULL;
   *outkeylen = *outvallen = 0;
 
+  if (NULL == indomain || '\0' == *indomain ||
+      NULL == inmap || '\0' == *inmap)
+    return YPERR_BADARGS;
+
   again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
@@ -509,43 +560,53 @@ yp_first (char *indomain, char *inmap, char **outkey, int *outkeylen, char **out
   yprnk.map = inmap;
   bzero ((char *) &yprkv, sizeof yprkv);
   
-  r = clnt_call (ysd->dom_client, YPPROC_FIRST,
-                 xdr_ypreq_nokey, &yprnk,
-                 xdr_ypresp_key_val, &yprkv,
-                 tv);
-  if (r != RPC_SUCCESS)
+  result = clnt_call (ysd->dom_client, YPPROC_FIRST,
+                      (xdrproc_t)xdr_ypreq_nokey, (char *)&yprnk,
+                      (xdrproc_t)xdr_ypresp_key_val, (char *)&yprkv,
+                      tv);
+  if (result != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_first: clnt_call");
       ysd->dom_vers = -1;
-      goto again;
+      __yp_unbind (ysd);
+      if (tries++ < __yplib_maxtries)
+        goto again;
+      return YPERR_RPC;
     }
-  if (!(r = ypprot_err (yprkv.status)))
+  result = ypprot_err (yprkv.status);
+  if (0 == result)
     {
-      *outkeylen = yprkv.keydat.dsize;
+      *outkeylen = yprkv.keydat.keydat_len;
       *outkey = (char *) malloc (*outkeylen + 2);
-      bcopy (yprkv.keydat.dptr, *outkey, *outkeylen);
+      bcopy (yprkv.keydat.keydat_val, *outkey, *outkeylen);
       (*outkey)[*outkeylen] = '\n';
       (*outkey)[*outkeylen + 1] = '\0';
-      *outvallen = yprkv.valdat.dsize;
+      *outvallen = yprkv.valdat.valdat_len;
       *outval = (char *) malloc (*outvallen + 2);
-      bcopy (yprkv.valdat.dptr, *outval, *outvallen);
+      bcopy (yprkv.valdat.valdat_val, *outval, *outvallen);
       (*outval)[*outvallen] = '\n';
       (*outval)[*outvallen + 1] = '\0';
     }
-  xdr_free (xdr_ypresp_key_val, (char *) &yprkv);
+  xdr_free ((xdrproc_t)xdr_ypresp_key_val, (char *) &yprkv);
   __yp_unbind (ysd);
-  return r;
+  return result;
 }
 
 int
-yp_next (char *indomain, char *inmap, char *inkey, int inkeylen, char **outkey, int *outkeylen, char **outval, int *outvallen)
+yp_next (char *indomain, char *inmap, char *inkey, int inkeylen,
+         char **outkey, int *outkeylen, char **outval, int *outvallen)
 {
   struct ypresp_key_val yprkv;
   struct ypreq_key yprk;
   struct dom_binding *ysd;
   struct timeval tv;
-  int r;
+  int result, tries = 0;
   
+  if (NULL == indomain || '\0' == *indomain ||
+      NULL == inmap || '\0' == *inmap ||
+      NULL == inkey || 0 == inkeylen || '\0' == *inkey)
+    return YPERR_BADARGS;
+
   *outkey = *outval = NULL;
   *outkeylen = *outvallen = 0;
   
@@ -558,34 +619,40 @@ yp_next (char *indomain, char *inmap, char *inkey, int inkeylen, char **outkey,
   
   yprk.domain = indomain;
   yprk.map = inmap;
-  yprk.keydat.dptr = inkey;
-  yprk.keydat.dsize = inkeylen;
+  yprk.keydat.keydat_val = inkey;
+  yprk.keydat.keydat_len = inkeylen;
   bzero ((char *) &yprkv, sizeof yprkv);
   
-  r = clnt_call (ysd->dom_client, YPPROC_NEXT,
-                 xdr_ypreq_key, &yprk, xdr_ypresp_key_val, &yprkv, tv);
-  if (r != RPC_SUCCESS)
+  result = clnt_call (ysd->dom_client, YPPROC_NEXT,
+                      (xdrproc_t)xdr_ypreq_key, (char *)&yprk,
+                      (xdrproc_t)xdr_ypresp_key_val, (char *)&yprkv,
+                      tv);
+  if (result != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_next: clnt_call");
       ysd->dom_vers = -1;
-      goto again;
+      __yp_unbind(ysd);
+      if (tries++ < __yplib_maxtries)
+        goto again;
+      return YPERR_RPC;
     }
-  if (!(r = ypprot_err (yprkv.status)))
+  result = ypprot_err (yprkv.status);
+  if (0 == result)
     {
-      *outkeylen = yprkv.keydat.dsize;
+      *outkeylen = yprkv.keydat.keydat_len;
       *outkey = (char *) malloc (*outkeylen + 2);
-      bcopy (yprkv.keydat.dptr, *outkey, *outkeylen);
+      bcopy (yprkv.keydat.keydat_val, *outkey, *outkeylen);
       (*outkey)[*outkeylen] = '\n';
       (*outkey)[*outkeylen + 1] = '\0';
-      *outvallen = yprkv.valdat.dsize;
+      *outvallen = yprkv.valdat.valdat_len;
       *outval = (char *) malloc (*outvallen + 2);
-      bcopy (yprkv.valdat.dptr, *outval, *outvallen);
+      bcopy (yprkv.valdat.valdat_val, *outval, *outvallen);
       (*outval)[*outvallen] = '\n';
       (*outval)[*outvallen + 1] = '\0';
     }
-  xdr_free (xdr_ypresp_key_val, (char *) &yprkv);
+  xdr_free ((xdrproc_t)xdr_ypresp_key_val, (char *) &yprkv);
   __yp_unbind (ysd);
-  return r;
+  return result;
 }
 
 int
@@ -599,6 +666,10 @@ yp_all (char *indomain, char *inmap, struct ypall_callback *incallback)
   u_long status;
   int clnt_sock;
   
+  if (NULL == indomain || '\0' == *indomain ||
+      NULL == inmap || '\0' == *inmap)
+    return YPERR_BADARGS;
+
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
   
@@ -610,19 +681,21 @@ yp_all (char *indomain, char *inmap, struct ypall_callback *incallback)
   clnt = clnttcp_create (&clnt_sin, YPPROG, YPVERS, &clnt_sock, 0, 0);
   if (NULL == clnt)
     {
-      printf ("clnttcp_create failed\n");
+      perror("clnttcp_create failed\n");
       return YPERR_PMAP;
     }
   
   yprnk.domain = indomain;
   yprnk.map = inmap;
-  ypresp_allfn = incallback->foreach;
-  ypresp_data = (void *) incallback->data;
+  ypresp_allfn = (void *)incallback->foreach;
+  ypresp_data = (char *) incallback->data;
   
   (void) clnt_call (clnt, YPPROC_ALL,
-		  xdr_ypreq_nokey, &yprnk, xdr_ypresp_all_seq, &status, tv);
+		  (xdrproc_t)xdr_ypreq_nokey, (char *)&yprnk,
+        (xdrproc_t)xdr_ypresp_all_seq, (char *)&status, tv);
   clnt_destroy (clnt);
-  xdr_free (xdr_ypresp_all_seq, (char *) &status);	/* not really needed... */
+   	/* not really needed... */
+  xdr_free ((xdrproc_t)xdr_ypresp_all_seq, (char *) &status);
   __yp_unbind (ysd);
   
   if (status != YP_FALSE)
@@ -637,8 +710,12 @@ yp_order (char *indomain, char *inmap, int *outorder)
   struct ypresp_order ypro;
   struct ypreq_nokey yprnk;
   struct timeval tv;
-  int r;
+  int result, tries = 0;
   
+  if (NULL == indomain || '\0' == *indomain ||
+      NULL == inmap || '\0' == *inmap)
+    return YPERR_BADARGS;
+
   again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
@@ -651,17 +728,22 @@ yp_order (char *indomain, char *inmap, int *outorder)
   
   bzero ((char *) (char *) &ypro, sizeof ypro);
   
-  r = clnt_call (ysd->dom_client, YPPROC_ORDER,
-                 xdr_ypreq_nokey, &yprnk, xdr_ypresp_order, &ypro, tv);
-  if (r != RPC_SUCCESS)
+  result = clnt_call (ysd->dom_client, YPPROC_ORDER,
+                      (xdrproc_t)xdr_ypreq_nokey, (char *)&yprnk,
+                      (xdrproc_t)xdr_ypresp_order, (char *)&ypro,
+                      tv);
+  if (result != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_order: clnt_call");
       ysd->dom_vers = -1;
-      goto again;
+      __yp_unbind(ysd);
+      if (tries++ < __yplib_maxtries)
+        goto again;
+      return YPERR_RPC;
     }
-
+  
   *outorder = ypro.ordernum;
-  xdr_free (xdr_ypresp_order, (char *) &ypro);
+  xdr_free ((xdrproc_t)xdr_ypresp_order, (char *) &ypro);
   __yp_unbind (ysd);
   return ypprot_err (ypro.status);
 }
@@ -673,8 +755,12 @@ yp_master (char *indomain, char *inmap, char **outname)
   struct ypresp_master yprm;
   struct ypreq_nokey yprnk;
   struct timeval tv;
-  int r;
+  int result, tries = 0;
   
+  if (NULL == indomain || '\0' == *indomain ||
+      NULL == inmap || '\0' == *inmap)
+    return YPERR_BADARGS;
+
   again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
@@ -687,21 +773,27 @@ yp_master (char *indomain, char *inmap, char **outname)
   
   bzero ((char *) &yprm, sizeof yprm);
 
-  r = clnt_call (ysd->dom_client, YPPROC_MASTER,
-                 xdr_ypreq_nokey, &yprnk, xdr_ypresp_master, &yprm, tv);
-  if (r != RPC_SUCCESS)
+  result = clnt_call (ysd->dom_client, YPPROC_MASTER,
+                      (xdrproc_t)xdr_ypreq_nokey, (char *)&yprnk,
+                      (xdrproc_t)xdr_ypresp_master, (char *)&yprm,
+                      tv);
+  if (result != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_master: clnt_call");
       ysd->dom_vers = -1;
-      goto again;
+      __yp_unbind(ysd);
+      if (tries++ < __yplib_maxtries)
+        goto again;
+      return YPERR_RPC;
     }
-  if (!(r = ypprot_err (yprm.status)))
+  result = ypprot_err (yprm.status);
+  if (0 == result)
     {
       *outname = (char *) strdup (yprm.master);
     }
-  xdr_free (xdr_ypresp_master, (char *) &yprm);
+  xdr_free ((xdrproc_t)xdr_ypresp_master, (char *) &yprm);
   __yp_unbind (ysd);
-  return r;
+  return result;
 }
 
 int
@@ -710,8 +802,11 @@ yp_maplist (char *indomain, struct ypmaplist **outmaplist)
   struct dom_binding *ysd;
   struct ypresp_maplist ypml;
   struct timeval tv;
-  int r;
+  int result, tries = 0;
   
+  if (NULL == indomain || '\0' == *indomain)
+    return YPERR_BADARGS;
+
   again:
   if (__yp_dobind (indomain, &ysd) != 0)
     return YPERR_DOMAIN;
@@ -721,13 +816,18 @@ yp_maplist (char *indomain, struct ypmaplist **outmaplist)
   
   bzero ((char *) &ypml, sizeof ypml);
   
-  r = clnt_call (ysd->dom_client, YPPROC_MAPLIST,
-                 xdr_domainname, indomain, xdr_ypresp_maplist, &ypml, tv);
-  if (r != RPC_SUCCESS)
+  result = clnt_call (ysd->dom_client, YPPROC_MAPLIST,
+                      (xdrproc_t)xdr_domainname, indomain,
+                      (xdrproc_t)xdr_ypresp_maplist, (char *)&ypml,
+                      tv);
+  if (result != RPC_SUCCESS)
     {
       clnt_perror (ysd->dom_client, "yp_maplist: clnt_call");
       ysd->dom_vers = -1;
-      goto again;
+      __yp_unbind(ysd);
+      if (tries++ < __yplib_maxtries)
+        goto again;
+      return YPERR_RPC;
     }
   *outmaplist = ypml.list;
       /* NO: xdr_free(xdr_ypresp_maplist, &ypml);*/
@@ -820,6 +920,14 @@ __yp_check (char **dom)
   if ('\0' == __yp_domain[0])
     if (yp_get_default_domain (&unused))
       return 0;
+#ifdef __linux__
+  /* If you didn't specify the domain name while building the Linux
+   * kernel, your domain name will be set to "(none)".
+   */
+    else
+      if (strcmp(__yp_domain,"(none)") == 0)
+	return 0;
+#endif
   
   if (dom)
     *dom = __yp_domain;
